from database import app
from covid19.blueprints.app_all.all_service_config import ApplicationServiceConfig
from covid19.blueprints.app_all.all_service_download import ApplicationServiceDownload
from covid19.blueprints.ecdc.ecdc_service_import import EcdcServiceImport
from covid19.blueprints.ecdc.ecdc_service_update import EcdcServiceUpdate, EcdcServiceUpdateFull


class EcdcService:
    def __init__(self, database):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" ECDC Service [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = ApplicationServiceConfig.create_config_for_ecdc()
        self.service_download = ApplicationServiceDownload(database, self.cfg)
        self.service_import = EcdcServiceImport(database, self.cfg)
        self.service_update = EcdcServiceUpdate(database, self.cfg)
        self.service_update_full = EcdcServiceUpdateFull(database, self.cfg)
        app.logger.debug("------------------------------------------------------------")
        app.logger.info(" ECDC Service [ready] ")

    def pretask_database_drop_create(self):
        self.service_download.download_file()
        return self

    def task_database_drop_create(self):
        self.service_import.import_file()
        self.service_update_full.full_update_star_schema()
        return self

    def run_download_only(self):
        self.service_download.download_file()
        return self

    def run_import_only(self):
        self.service_import.import_file()
        return self

    def run_full_update_dimension_tables(self):
        self.service_update_full.full_update_dimension_tables()
        return self

    def run_update_dimension_tables(self):
        self.service_update.update_dimension_tables()
        return self

    def run_update_fact_table(self):
        self.service_update.update_fact_table()
        return self

    def run_full_update_fact_table(self):
        self.service_update_full.full_update_fact_table()
        return self

    def run_update_star_schema(self):
        self.service_import.import_file()
        self.service_update.update_star_schema()
        return self

    def task_admin_full_update_star_schema(self):
        self.service_import.import_file()
        self.service_update_full.full_update_star_schema()
        return self

    def download_all_files(self):
        self.service_download.download_file()
        return self

    def task_admin_import_all_files(self):
        self.service_import.import_file()
        return self

    def task_admin_update_dimension_tables(self):
        self.service_update.update_dimension_tables()
        return self

    def update_fact_table_initial_only(self):
        self.service_update_full.full_update_fact_table()
        return self

    def update_fact_table_incremental_only(self):
        self.service_update.update_fact_table()
        return self
