from flask import render_template, redirect, url_for, flash, Blueprint
from celery import states
from celery.utils.log import get_task_logger
from flask_login import login_required

from database import app, db
from covid19.blueprints.app_all.all_services import who_service, owid_service
from covid19.blueprints.app_all.all_services import rki_service, rki_vaccination_service
from covid19.blueprints.app_all.all_services import ecdc_service, divi_service
from covid19.blueprints.app_all.all_services import app_admin_service, app_user_service
from covid19.blueprints.app_mq.application_workers import celery
from covid19.blueprints.app_application.application_model_transient import ApplicationPage

from covid19.blueprints.ecdc.ecdc_model_import import EcdcImport
from covid19.blueprints.ecdc_test.ecdc_test_service import EcdcTestService

ecdc_test_service = EcdcTestService(db, ecdc_service)

app_ecdc_test = Blueprint('ecdc_test', __name__, template_folder='templates', url_prefix='/ecdc/test')

# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------


@app_ecdc_test.route('/tests')
@login_required
def url_ecdc_test_tests():
    page_info = ApplicationPage('ECDC', "Tests")
    return render_template(
        'ecdc_test/ecdc_tests.html',
        page_info=page_info)


@app_ecdc_test.route('/ecdc_import/countries')
@login_required
def url_ecdc_test_ecdc_import_countries():
    flash("url_ecdc_test_ecdc_import_countries - START: EcdcImport.countries()")
    app.logger.info("url_ecdc_test_ecdc_import_countries - START: EcdcImport.countries()")
    i = 0
    for c in EcdcImport.countries():
        i += 1
        line = " | " + str(i) + " | " + c.countries.country_code + " | " + c.countries.country + " | " + c.countries.ecdc_region + " | "
        app.logger.info(line)
    flash("url_ecdc_test_ecdc_import_countries - DONE: EcdcImport.countries()")
    return redirect(url_for('ecdc_test.url_ecdc_test_tests'))


@app_ecdc_test.route('/ecdc_import/get_new_dates_as_array')
@login_required
def url_ecdc_test_ecdc_import_get_new_dates_as_array():
    app.logger.info("url_ecdc_mytest - DONE: EcdcImport.countries()")
    flash("url_ecdc_mytest - START: EcdcImport.get_new_dates_as_array()")
    app.logger.info("url_ecdc_mytest - START: EcdcImport.get_new_dates_as_array()")
    app.logger.info("EcdcImport.get_new_dates_as_array():")
    i = 0
    for date_reported in EcdcImport.get_new_dates_as_array():
        i += 1
        line = " | " + str(i) + " | " + date_reported + " | "
        app.logger.info(line)
    flash("url_ecdc_mytest - DONE: EcdcImport.get_new_dates_as_array()")
    app.logger.info("url_ecdc_mytest - DONE: EcdcImport.get_new_dates_as_array()")
    return redirect(url_for('ecdc_test.url_ecdc_test_tests'))


@app_ecdc_test.route('/ecdc_data/get_datum_of_all_ecdc_data')
@login_required
def url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_data():
    app.logger.info("url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_data - DONE: EcdcData.get_datum_of_all_ecdc_data()")
    flash("url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_data - START: EcdcData.get_datum_of_all_ecdc_data()")
    for datum in EcdcImport.get_datum_of_all_data():
        app.logger.info(str(datum))
    flash("url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_data - DONE: EcdcData.get_datum_of_all_ecdc_data()")
    app.logger.info("url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_data - DONE: EcdcData.get_datum_of_all_ecdc_data()")
    return redirect(url_for('ecdc_test.url_ecdc_test_tests'))


@app_ecdc_test.route('/ecdc_data/get_datum_of_all_ecdc_import')
@login_required
def url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_import():
    app.logger.info("url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_import - START: EcdcImport.get_datum_of_all_ecdc_import()")
    flash("url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_import - START: EcdcImport.get_datum_of_all_ecdc_import()")
    for datum in EcdcImport.get_datum_of_all_ecdc_import():
        app.logger.info(str(datum))
    flash("url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_import - DONE: EcdcImport.get_datum_of_all_ecdc_import()")
    app.logger.info("url_ecdc_test_ecdc_data_get_datum_of_all_ecdc_import - DONE: EcdcImport.get_datum_of_all_ecdc_import()")
    return redirect(url_for('ecdc_test.url_ecdc_test_tests'))


@app_ecdc_test.route('/ecdc_service/service_update/ecdc_import_get_new_dates_as_array')
@login_required
def url_ecdc_test_ecdc_service_ecdc_import_get_new_dates_as_array():
    app.logger.info("url_ecdc_test_ecdc_import_get_new_dates_as_array - START: EcdcService.ecdc_import_get_new_dates_as_array()")
    flash("url_ecdc_test_ecdc_import_get_new_dates_as_array - START: EcdcService.ecdc_import_get_new_dates_as_array()")
    for datum in ecdc_service.service_update.ecdc_import_get_new_dates_as_array():
        app.logger.info(str(datum))
    flash("url_ecdc_test_ecdc_import_get_new_dates_as_array - DONE: EcdcService.ecdc_import_get_new_dates_as_array()")
    app.logger.info("url_ecdc_test_ecdc_import_get_new_dates_as_array - DONE: EcdcService.ecdc_import_get_new_dates_as_array()")
    return redirect(url_for('ecdc_test.url_ecdc_test_tests'))


@app_ecdc_test.route('/ecdc_test_service/delete_last_day')
@login_required
def url_ecdc_test_ecdc_test_service_delete_last_days_data():
    app.logger.info("url_ecdc_test_ecdc_test_service_delete_last_days_data - START: EcdcService.ecdc_import_get_new_dates_as_array()")
    flash("url_ecdc_test_ecdc_test_service_delete_last_days_data - START: EcdcService.ecdc_import_get_new_dates_as_array()")
    ecdc_test_service.delete_last_day()
    flash("url_ecdc_test_ecdc_test_service_delete_last_days_data - DONE: EcdcService.ecdc_import_get_new_dates_as_array()")
    app.logger.info("url_ecdc_test_ecdc_test_service_delete_last_days_data - DONE: EcdcService.ecdc_import_get_new_dates_as_array()")
    return redirect(url_for('ecdc_test.url_ecdc_test_tests'))


# ----------------------------------------------------------------------------------------------------------------
#  Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@celery.task(bind=True)
def task_ecdc_test_update_star_schema_incremental(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_test_update_star_schema_incremental [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_test_service.run_update_star_schema_incremental()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_test_update_star_schema_incremental)"
    return result

# ----------------------------------------------------------------------------------------------------------------
#  URL Routes for Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@app_ecdc_test.route('/task/update_star_schema_incremental')
@login_required
def url_task_ecdc_test_update_star_schema_incremental():
    app.logger.info("url_task_ecdc_test_update_star_schema_incremental - START: task_ecdc_test_update_star_schema_incremental()")
    flash("url_task_ecdc_test_update_star_schema_incremental - START: task_ecdc_test_update_star_schema_incremental()")
    task_ecdc_test_update_star_schema_incremental.apply_async()
    flash("url_task_ecdc_test_update_star_schema_incremental - DONE: task_ecdc_test_update_star_schema_incremental()")
    app.logger.info("url_task_ecdc_test_update_star_schema_incremental - DONE: task_ecdc_test_update_star_schema_incremental()")
    return redirect(url_for('ecdc_test.url_ecdc_test_tests'))
