from flask import flash

from database import app
from covid19.blueprints.app_all.all_service_config import ApplicationServiceConfig
from covid19.blueprints.app_all.all_service_download import ApplicationServiceDownload
from covid19.blueprints.intensivregister.divi_service_import import DiviServiceImport
from covid19.blueprints.intensivregister.divi_service_update import DiviServiceUpdate


class DiviService:
    def __init__(self, database):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" DIVI Service [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = ApplicationServiceConfig.create_config_for_intensivregister()
        self.service_download = ApplicationServiceDownload(database, self.cfg)
        self.service_import = DiviServiceImport(database, self.cfg)
        self.service_update = DiviServiceUpdate(database, self.cfg)
        app.logger.debug("------------------------------------------------------------")
        app.logger.info(" DIVI Service [ready]")

    def pretask_database_drop_create(self):
        flash("DiviService.pretask_database_drop_create started")
        app.logger.info("DiviService.pretask_database_drop_create started")
        self.service_download.download_file()
        app.logger.info("DiviService.pretask_database_drop_create done")
        flash("DiviService.pretask_database_drop_create done")
        return self

    def task_database_drop_create(self):
        app.logger.info("DiviService.task_database_drop_create started")
        self.service_import.import_file()
        self.service_update.update_dimension_tables_only()
        self.service_update.update_fact_table_incremental_only()
        app.logger.info("DiviService.task_database_drop_create done")
        return self

    #def run_download_only(self):
    #def run_import_only(self):
    #def run_update_dimension_tables_only(self):
    #def run_update_fact_table_incremental_only(self):
    #def run_update_fact_table_initial_only(self):
    #def run_update_star_schema_incremental(self):
    #def run_update_star_schema_initial(self):

    def run_download_only(self):
        self.service_download.download_file()
        return self

    def run_import_only(self):
        self.service_import.import_file()
        return self

    def run_update_dimension_tables_only(self):
        self.service_update.update_dimension_tables_only()
        return self

    def run_update_fact_table_incremental_only(self):
        self.service_update.update_fact_table_incremental_only()
        return self

    def run_update_fact_table_initial_only(self):
        self.service_update.update_fact_table_initial_only()
        return self

    def run_update_star_schema_incremental(self):
        self.service_import.import_file()
        self.service_update.update_star_schema_incremental()
        return self

    def task_admin_full_update_star_schema(self):
        self.service_import.import_file()
        self.service_update.update_star_schema_initial()
        return self

    def download_all_files(self):
        self.service_download.download_file()
        return self

    def task_import_all_files(self):
        self.service_import.import_file()
        return self

    def update_dimension_tables_only(self):
        self.service_update.update_dimension_tables_only()
        return self

    def update_fact_table_initial_only(self):
        self.service_update.update_fact_table_initial_only()
        return self

    def update_fact_table_incremental_only(self):
        self.service_update.update_fact_table_incremental_only()
        return self
