from flask import flash

from database import app

from covid19.blueprints.app_all.all_service_config import ApplicationServiceConfig
from covid19.blueprints.app_all.all_service_download import ApplicationServiceDownload
from covid19.blueprints.owid.owid_service_import import OwidServiceImport
from covid19.blueprints.owid.owid_service_update import OwidServiceUpdate, OwidServiceUpdateFull


class OwidService:
    def __init__(self, database):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OWID Service [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = ApplicationServiceConfig.create_config_for_owid()
        self.service_download = ApplicationServiceDownload(database, self.cfg)
        self.service_import = OwidServiceImport(database, self.cfg)
        self.service_update = OwidServiceUpdate(database, self.cfg)
        self.service_update_full = OwidServiceUpdateFull(database, self.cfg)
        app.logger.debug("------------------------------------------------------------")
        app.logger.info(" OWID Service [ready]")

    def download_all_files(self):
        app.logger.info("OwidService.pretask_database_drop_create started")
        self.service_download.download_file()
        app.logger.info("OwidService.pretask_database_drop_create done")
        return self

    def pretask_database_drop_create(self):
        flash("OwidService.pretask_database_drop_create started")
        app.logger.info("OwidService.pretask_database_drop_create started")
        self.service_download.download_file()
        app.logger.info("OwidService.pretask_database_drop_create done")
        flash("OwidService.pretask_database_drop_create done")
        return self

    def task_database_drop_create(self):
        app.logger.info("OwidService.task_database_drop_create started")
        self.service_import.import_file()
        # TODO #212 implement OwidService.task_database_drop_create()
        self.service_update.update_dimension_tables()
        self.service_update.update_fact_table()
        app.logger.info("OwidService.task_database_drop_create done")
        return self

    def run_download(self):
        self.service_download.download_file()
        return self

    def run_import(self):
        self.service_import.import_file()
        return self

    def task_admin_import_all_files(self):
        self.service_import.import_file()
        return self

    def update_star_schema(self):
        self.service_import.import_file()
        self.service_update.update_star_schema()
        return self

    def full_update_star_schema(self):
        self.service_import.import_file()
        self.service_update_full.full_update_star_schema()
        return self

    def task_admin_full_update_star_schema(self):
        self.service_import.import_file()
        self.service_update_full.full_update_star_schema()
        return self

    def task_admin_update_full_dimension_tables(self):
        self.service_update_full.full_update_dimension_tables()
        return self

    def task_admin_update_dimension_tables(self):
        self.service_update.update_dimension_tables()
        return self

