from database import db, app
from covid19.blueprints.app_all.all_service_config import ApplicationServiceConfig
from covid19.blueprints.owid.owid_model import OwidDateReported, OwidData, OwidContinent, OwidCountry
from covid19.blueprints.owid.owid_model_import import OwidImport


class OwidTestService:
    def __init__(self, database, owid_service):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.__owid_service = owid_service
        self.cfg = ApplicationServiceConfig.create_config_for_owid()
        app.logger.debug("------------------------------------------------------------")
        app.logger.info(" OwidTestService [ready]")

    def delete_last_day(self):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService.delete_last_day() [START]")
        app.logger.debug("------------------------------------------------------------")
        joungest_datum_str = OwidData.get_joungest_datum()
        joungest_datum = OwidDateReported.find_by_date_reported(joungest_datum_str)
        app.logger.info("joungest_datum:")
        app.logger.info(joungest_datum)
        app.logger.info("OwidData.get_data_for_one_day(joungest_datum):")
        i = 0
        for data in OwidData.get_data_for_one_day(joungest_datum):
            i += 1
            line = "Owid: to be deleted | " + str(i) + " | " + str(data.date_reported) + " | " + str(data.country) + " | "
            app.logger.info(line)
        app.logger.info("OwidData.delete_data_for_one_day(joungest_datum)")
        OwidData.delete_data_for_one_day(joungest_datum)
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService.delete_last_day() [DONE]")
        app.logger.debug("------------------------------------------------------------")

    def delete_last_continent(self):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService.delete_last_continent() [START]")
        app.logger.debug("------------------------------------------------------------")
        app.logger.info("OwidContinent.get_last_continent()")
        last_continent = OwidContinent.get_last_continent()
        app.logger.info("last_continent:" + str(last_continent))
        app.logger.info("OwidCountry.get_all_countries_for_continent(last_continent)")
        countries_for_continent = OwidCountry.get_all_countries_for_continent(last_continent)
        i = 0
        for country in countries_for_continent:
            for data in OwidData.get_all_data_for_country(country):
                i += 1
                line = "Owid: to be deleted | " + str(i) + " | " + str(data.date_reported) + " | " + str(data.country) + " | "
                app.logger.info(line)
            app.logger.info("OwidData.delete_all_data_for_country(country)")
            OwidData.delete_all_data_for_country(country)
        app.logger.info("OwidCountry.delete_all_countries_for_continent(last_continent)")
        OwidCountry.delete_all_countries_for_continent(last_continent)
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService.delete_last_continent() [DONE]")
        app.logger.debug("------------------------------------------------------------")

    def run_update_star_schema_incremental(self):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService.run_update_star_schema_incremental() [START]")
        app.logger.debug("------------------------------------------------------------")
        self.__owid_service.run_update_star_schema_incremental()
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService.run_update_star_schema_incremental() [DONE]")
        app.logger.debug("------------------------------------------------------------")

    def full_update_dimension_tables(self):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService.full_update_dimension_tables() [START]")
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug("")
        OwidData.remove_all()
        OwidCountry.remove_all()
        for continent in OwidContinent.get_all():
            app.logger.info("continent.region: " + continent.region)
            for oi in OwidImport.get_countries_for_continent(continent.region):
                app.logger.info("continent.region: " + continent.region +" - oi.location: " + oi.location)
                o = OwidCountry(
                    continent_id=continent.id,
                    continent=continent,
                    location=oi.location,
                    iso_code=oi.iso_code,
                    population=oi.population,
                    population_density=oi.population_density,
                    median_age=oi.median_age,
                    aged_65_older=oi.aged_65_older,
                    aged_70_older=oi.aged_70_older,
                    gdp_per_capita=oi.gdp_per_capita,
                    extreme_poverty=oi.extreme_poverty,
                    cardiovasc_death_rate=oi.cardiovasc_death_rate,
                    diabetes_prevalence=oi.diabetes_prevalence,
                    female_smokers=oi.female_smokers,
                    male_smokers=oi.male_smokers,
                    handwashing_facilities=oi.handwashing_facilities,
                    hospital_beds_per_thousand=oi.hospital_beds_per_thousand,
                    life_expectancy=oi.life_expectancy,
                    human_development_index=oi.human_development_index
                )
                db.session.add(o)
        db.session.commit()
        app.logger.debug("")
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" OwidTestService.full_update_dimension_tables() [DONE]")
        app.logger.debug("------------------------------------------------------------")
