from flask import render_template, redirect, url_for, flash, Blueprint
from celery import states
from celery.utils.log import get_task_logger
from flask_login import login_required

from database import app, db
from covid19.blueprints.app_all.all_services import who_service, owid_service
from covid19.blueprints.app_all.all_services import rki_service, rki_vaccination_service
from covid19.blueprints.app_all.all_services import ecdc_service, divi_service
from covid19.blueprints.app_all.all_services import app_admin_service, app_user_service
from covid19.blueprints.app_mq.application_workers import celery
from covid19.blueprints.owid.owid_model import OwidData
from covid19.blueprints.owid.owid_model_import import OwidImport
from covid19.blueprints.app_application.application_model_transient import ApplicationPage

from covid19.blueprints.owid_test.owid_test_service import OwidTestService

owid_test_service = OwidTestService(db, owid_service)

app_owid_test = Blueprint('owid_test', __name__, template_folder='templates', url_prefix='/owid/test')

# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------

@app_owid_test.route('/tests')
@login_required
def url_owid_test_tests():
    page_info = ApplicationPage('OWID', "Tests")
    return render_template(
        'owid_test/owid_tests.html',
        page_info=page_info)


@app_owid_test.route('/update_dimension_tables_only')
@login_required
def url_owid_test_update_dimension_tables_only():
    app.logger.info("test_update_dimension_tables_only - START")
    flash("test_update_dimension_tables_only - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/update_fact_table_incremental_only')
@login_required
def url_owid_test_update_fact_table_incremental_only():
    app.logger.info("update_fact_table_incremental_only - START")
    flash("update_fact_table_incremental_only - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/update_fact_table_initial_only')
@login_required
def url_owid_test_update_fact_table_initial_only():
    app.logger.info("update_fact_table_initial_only - START")
    flash("update_fact_table_initial_only - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/update_star_schema/incremental')
@login_required
def url_owid_test_update_star_schema_incremental():
    app.logger.info("update_star_schema_incremental - START")
    flash("update_star_schema_incremental - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/update_star_schema/initial')
@login_required
def url_owid_test_update_star_schema_initial():
    app.logger.info("update_star_schema_initial - START")
    flash("update_star_schema_initial - START")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/owid_import/countries')
@login_required
def url_owid_test_owid_import_countries():
    app.logger.info("url_owid_test_owid_import_countries - START")
    flash("url_owid_test_owid_import_countries - START")
    i = 0
    for c in OwidImport.countries():
        i += 1
        line = " | " + str(i) + " | " + c.countries.iso_code + " | " + c.countries.location + " | " + c.countries.continent + " | "
        app.logger.info(line)
    app.logger.info("url_owid_test_owid_import_countries - DONE")
    flash("url_owid_test_owid_import_countries - DONE")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/owid_import/get_new_dates_as_array')
@login_required
def url_owid_test_owid_import_get_new_dates_as_array():
    app.logger.info("url_owid_test_owid_import_get_new_dates_as_array - START")
    flash("url_owid_test_owid_import_get_new_dates_as_array - START")
    i = 0
    for date_reported in OwidImport.get_new_dates_reported_as_array():
        i += 1
        line = " | " + str(i) + " | " + date_reported + " | "
        app.logger.info(line)
    app.logger.info("url_owid_test_owid_import_get_new_dates_as_array - DONE")
    flash("url_owid_test_owid_import_get_new_dates_as_array - DONE")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/owid_data/get_datum_of_all_owid_data')
@login_required
def url_owid_test_owid_data_get_datum_of_all_owid_data():
    app.logger.info("url_owid_test_owid_data_get_datum_of_all_owid_data - START")
    flash("url_owid_test_owid_data_get_datum_of_all_owid_data - START")
    for datum in OwidData.get_datum_of_all_data():
        app.logger.info(str(datum))
    app.logger.info("url_owid_test_owid_data_get_datum_of_all_owid_data - DONE")
    flash("url_owid_test_owid_data_get_datum_of_all_owid_data - DONE")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/owid_data/update_star_schema_initial')
@login_required
def url_owid_test_owid_data_get_datum_of_all_owid_import():
    app.logger.info("url_owid_test_owid_data_get_datum_of_all_owid_import - START")
    flash("url_owid_test_owid_data_get_datum_of_all_owid_import - START")
    for datum in OwidImport.get_datum_of_all_import():
        app.logger.info(str(datum))
    app.logger.info("url_owid_test_owid_data_get_datum_of_all_owid_import - DONE")
    flash("url_owid_test_owid_data_get_datum_of_all_owid_import - DONE")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/owid_service/owid_import/get_new_dates_as_array')
@login_required
def url_owid_test_owid_service_owid_import_get_new_dates_as_array():
    app.logger.info("url_owid_test_owid_service_owid_import_get_new_dates_as_array - START")
    flash("url_owid_test_owid_service_owid_import_get_new_dates_as_array - START")
    for datum in owid_service.service_update.who_import_get_new_dates_as_array():
        app.logger.info(str(datum))
    app.logger.info("url_owid_test_owid_service_owid_import_get_new_dates_as_array - DONE")
    flash("url_owid_test_owid_service_owid_import_get_new_dates_as_array - DONE")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/owid_test_service/delete_last_days_data')
@login_required
def url_owid_test_owid_test_service_delete_last_days_data():
    app.logger.info("url_owid_test_owid_test_service_delete_last_days_data - START")
    flash("url_owid_test_owid_test_service_delete_last_days_data - START")
    owid_test_service.delete_last_day()
    app.logger.info("url_owid_test_owid_test_service_delete_last_days_data - DONE")
    flash("url_owid_test_owid_test_service_delete_last_days_data - DONE")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/owid_test_service/delete_last_continents_data')
@login_required
def url_owid_test_owid_test_service_delete_last_continents_data():
    app.logger.info("url_owid_test_owid_test_service_delete_last_continents_data - START")
    flash("owid_test_service.delete_last_continent() - START")
    owid_test_service.delete_last_continent()
    flash("owid_test_service.delete_last_continent() - DONE")
    app.logger.info("url_owid_test_owid_test_service_delete_last_continents_data - DONE")
    return redirect(url_for('owid_test.url_owid_test_tests'))


@app_owid_test.route('/owid_test_service/update_dimension_tables')
@login_required
def url_owid_test_full_update_dimension_tables():
    app.logger.info("url_owid_test_full_update_dimension_tables - START")
    flash("owid_test_service.full_update_dimension_tables() - START")
    owid_test_service.full_update_dimension_tables()
    app.logger.info("owid_test_service.full_update_dimension_tables() - DONE")
    flash("url_owid_test_full_update_dimension_tables - DONE")
    return redirect(url_for('owid_test.url_owid_test_tests'))


# ----------------------------------------------------------------------------------------------------------------
#  Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@celery.task(bind=True)
def task_owid_test_update_star_schema_incremental(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_owid_test_update_star_schema_incremental [OK] ")
    logger.info("------------------------------------------------------------")
    owid_test_service.run_update_star_schema_incremental()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_owid_test_update_star_schema_incremental)"
    return result

# ----------------------------------------------------------------------------------------------------------------
#  URL Routes for Celery TASKS
# ----------------------------------------------------------------------------------------------------------------

@app_owid_test.route('/task/owid_test/update_star_schema_incremental')
@login_required
def url_task_owid_test_update_star_schema_incremental():
    app.logger.info("url_task_owid_test_update_star_schema_incremental - START: task_owid_test_update_star_schema_incremental()")
    flash("url_task_owid_test_update_star_schema_incremental - START: task_owid_test_update_star_schema_incremental()")
    task_owid_test_update_star_schema_incremental.apply_async()
    flash("url_task_owid_test_update_star_schema_incremental - DONE: task_owid_test_update_star_schema_incremental()")
    app.logger.info("url_task_owid_test_update_star_schema_incremental - DONE: task_owid_test_update_star_schema_incremental()")
    return redirect(url_for('owid_test.url_owid_test_tests'))






