from flask import flash

from database import app
from covid19.blueprints.app_all.all_service_config import ApplicationServiceConfig
from covid19.blueprints.app_all.all_service_download import ApplicationServiceDownload
from covid19.blueprints.rki_cases.rki_service_import import RkiServiceImport
from covid19.blueprints.rki_cases.rki_service_update import RkiServiceUpdate, RkiServiceUpdateFull


class RkiService:
    def __init__(self, database):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RKI Service [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = ApplicationServiceConfig.create_config_for_rki()
        self.service_download = ApplicationServiceDownload(database, self.cfg)
        self.service_import = RkiServiceImport(database, self.cfg)
        self.service_update = RkiServiceUpdate(database, self.cfg)
        self.service_update_full = RkiServiceUpdateFull(database, self.cfg)
        app.logger.debug("------------------------------------------------------------")
        app.logger.info(" RKI Service [ready]")

    def pretask_database_drop_create(self):
        flash("RkiService.pretask_database_drop_create start")
        app.logger.info("RkiService.pretask_database_drop_create start")
        self.service_download.download_file()
        app.logger.info("RkiService.pretask_database_drop_create done")
        flash("RkiService.pretask_database_drop_create done")
        return self

    def task_database_drop_create(self):
        app.logger.info("RkiService.task_database_drop_create start")
        self.service_import.import_file()
        self.service_update_full.full_update_star_schema()
        app.logger.info("RkiService.task_database_drop_create done")
        return self

    def run_download(self):
        flash("RkiService.run_download start")
        self.service_download.download_file()
        flash("RkiService.run_download done")
        return self

    def run_import(self):
        self.service_import.import_file()
        return self

    def run_full_update_dimension_tables(self):
        self.service_update_full.full_update_dimension_tables()
        return self

    def run_update_dimension_tables(self):
        self.service_update.update_dimension_tables()
        return self

    def run_full_update_fact_table(self):
        self.service_update_full.full_update_fact_table()
        return self

    def run_update_fact_table(self):
        self.service_update.update_fact_table()
        return self

    def task_admin_full_update_star_schema(self):
        self.service_import.import_file()
        self.service_update_full.full_update_star_schema()
        return self

    def task_admin_update_star_schema(self):
        self.service_import.import_file()
        self.service_update.update_star_schema()
        return self

    def run_full_update_star_schema(self):
        self.service_import.import_file()
        self.service_update_full.full_update_star_schema()
        return self

    def run_update_star_schema(self):
        self.service_import.import_file()
        self.service_update.update_star_schema()
        return self

    def download_all_files(self):
        self.service_download.download_file()
        return self

    def task_import_all_files(self):
        self.service_import.import_file()
        return self

    def full_update_dimension_tables(self):
        self.service_update_full.full_update_dimension_tables()
        return self

    def update_dimension_tables(self):
        self.service_update.update_dimension_tables()
        return self

    def full_update_fact_table(self):
        self.service_update_full.full_update_fact_table()
        return self

    def update_fact_table(self):
        self.service_update.update_fact_table()
        return self
