import csv
import psycopg2

from database import db, app
from covid19.blueprints.app_all.all_service_config import ApplicationServiceConfig
from covid19.blueprints.rki_cases.rki_model_import import RkiImport


class RkiServiceImport:
    def __init__(self, database, config: ApplicationServiceConfig):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RKI Service Import [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = config
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" RKI Service Import [ready]")

    # TODO: BUG: RkiBundeslaenderServiceImport.import_file #177
    def import_file(self):
        app.logger.info(" import RKI [begin]")
        app.logger.info("------------------------------------------------------------")
        app.logger.info(" import into TABLE: "+self.cfg.tablename+" from "+self.cfg.cvsfile_path)
        app.logger.info("------------------------------------------------------------")
        row = None
        try:
            RkiImport.remove_all()
            with open(self.cfg.cvsfile_path, newline='\n') as csv_file:
                file_reader = csv.DictReader(csv_file, delimiter=',', quotechar='"')
                k = 0
                for row in file_reader:
                    k = k + 1
                    o = RkiImport(
                        fid=row['FID'],
                        id_bundesland=row['IdBundesland'],
                        bundesland=row['Bundesland'],
                        landkreis=row['Landkreis'],
                        altersgruppe=row['Altersgruppe'],
                        geschlecht=row['Geschlecht'],
                        anzahl_fall=row['AnzahlFall'],
                        anzahl_todesfall=row['AnzahlTodesfall'],
                        meldedatum=row['Meldedatum'],
                        id_landkreis=row['IdLandkreis'],
                        datenstand=row['Datenstand'],
                        neuer_fall=row['NeuerFall'],
                        neuer_todesfall=row['NeuerTodesfall'],
                        ref_datum=row['Refdatum'],
                        neu_genesen=row['NeuGenesen'],
                        anzahl_genesen=row['AnzahlGenesen'],
                        ist_erkrankungsbeginn=row['IstErkrankungsbeginn'],
                        altersgruppe2=row['Altersgruppe2']
                    )
                    db.session.add(o)
                    if (k % 2000) == 0:
                        db.session.commit()
                        app.logger.info(" import RKI ... " + str(k) + " rows")
                db.session.commit()
                app.logger.info(" import RKI ... " + str(k) + " rows total")
        except KeyError as error:
            app.logger.warning("WARN: import RKI [begin]")
            app.logger.warning(":::"+str(error)+":::")
            for item_key, item_value in row.items():
                app.logger.warning(item_key+" : "+item_value)
            app.logger.warning("WARN: import RKI Bundeslaender [end]")
        except (Exception, psycopg2.DatabaseError) as error:
            app.logger.warning("WARN: import RKI Bundeslaender [begin]")
            app.logger.warning(error)
            app.logger.warning("WARN: import RKI Bundeslaender [end]")
        finally:
            app.logger.info("")
            app.logger.info("------------------------------------------------------------")
            app.logger.info(" imported into TABLE: "+self.cfg.tablename+" from "+self.cfg.cvsfile_path)
            app.logger.info("------------------------------------------------------------")
            app.logger.info(" import RKI Bundeslaender  [done]")
        return self
