from flask import render_template, redirect, url_for, flash, Blueprint
from celery import states
from celery.utils.log import get_task_logger
from flask_login import login_required

from database import app, db

from covid19.blueprints.app_all.all_services import who_service, owid_service
from covid19.blueprints.app_all.all_services import rki_service, rki_vaccination_service
from covid19.blueprints.app_all.all_services import ecdc_service, divi_service
from covid19.blueprints.app_all.all_services import app_admin_service, app_user_service
from covid19.blueprints.app_mq.application_workers import celery
from covid19.blueprints.rki_cases.rki_model import RkiData
from covid19.blueprints.rki_cases.rki_model_import import RkiImport
from covid19.blueprints.app_application.application_model_transient import ApplicationPage

from covid19.blueprints.owid_test.owid_test_service import OwidTestService

rki_test_service = OwidTestService(db, rki_service)

app_rki_test = Blueprint('rki_test', __name__, template_folder='templates', url_prefix='/rki/test')

# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------


@app_rki_test.route('/tests')
@login_required
def url_rki_test_tests():
    app.logger.info("url_rki_test_tests - START")
    page_info = ApplicationPage('RKI', "Tests")
    return render_template(
        'rki_test/rki_tests.html',
        page_info=page_info)


@app_rki_test.route('/full_update_dimension_tables')
@login_required
def url_rki_test_full_update_dimension_tables():
    app.logger.info("url_rki_test_full_update_dimension_tables - START")
    flash("url_rki_test_full_update_dimension_tables - START")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/update_dimension_tables')
@login_required
def url_rki_test_update_dimension_tables():
    app.logger.info("url_rki_test_update_dimension_tables - START")
    flash("url_rki_test_update_dimension_tables - START")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/full_update_fact_table')
@login_required
def url_rki_test_full_update_fact_table():
    app.logger.info("url_rki_test_full_update_fact_table - START")
    flash("url_rki_test_full_update_fact_table - START")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/update_fact_table')
@login_required
def url_rki_test_update_fact_table():
    app.logger.info("url_rki_test_update_fact_table - START")
    flash("url_rki_test_update_fact_table - START")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/full_update_star_schema')
@login_required
def url_rki_test_full_update_star_schema():
    app.logger.info("url_rki_test_full_update_star_schema - START")
    flash("url_rki_test_full_update_star_schema - START")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/update_star_schema')
@login_required
def url_rki_test_update_star_schema():
    app.logger.info("url_rki_test_update_star_schema - START")
    flash("url_rki_test_update_star_schema - START")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/rki_import/countries')
@login_required
def url_rki_test_rki_import_countries():
    app.logger.info("url_rki_test_rki_import_countries - START")
    flash("url_rki_test_rki_import_countries - START")
    i = 0
    for c in RkiImport.countries():
        i += 1
        line = " | " + str(i) + " | " + c.countries.iso_code + " | " + c.countries.location + " | " + c.countries.continent + " | "
        app.logger.info(line)
    app.logger.info("url_rki_test_rki_import_countries - DONE")
    flash("url_rki_test_rki_import_countries - DONE")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/rki_import/get_new_dates_reported_as_array')
@login_required
def url_rki_test_rki_import_get_new_dates_reported_as_array():
    app.logger.info("url_rki_test_rki_import_get_new_dates_reported_as_array - START")
    flash("url_rki_test_rki_import_get_new_dates_reported_as_array - START")
    i = 0
    for date_reported in RkiImport.get_new_dates_reported_as_array():
        i += 1
        line = " | " + str(i) + " | " + date_reported + " | "
        app.logger.info(line)
    app.logger.info("url_rki_test_rki_import_get_new_dates_reported_as_array - DONE")
    flash("url_rki_test_rki_import_get_new_dates_reported_as_array - DONE")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/rki_data/get_datum_of_all_data')
@login_required
def url_rki_test_rki_data_get_datum_of_all_data():
    app.logger.info("url_rki_test_rki_data_get_datum_of_all_data - START")
    flash("url_rki_test_rki_data_get_datum_of_all_data - START")
    for datum in RkiData.get_datum_of_all_data():
        app.logger.info(str(datum))
    app.logger.info("url_rki_test_rki_data_get_datum_of_all_data - DONE")
    flash("url_rki_test_rki_data_get_datum_of_all_data - DONE")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/rki_import/get_datum_of_all_import')
@login_required
def url_rki_test_rki_import_get_datum_of_all_import():
    app.logger.info("url_rki_test_rki_import_get_datum_of_all_import - START")
    flash("url_rki_test_rki_import_get_datum_of_all_import - START")
    for datum in RkiImport.get_datum_of_all_import():
        app.logger.info(str(datum))
    app.logger.info("url_rki_test_rki_import_get_datum_of_all_import - DONE")
    flash("url_rki_test_rki_import_get_datum_of_all_import - DONE")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/rki_service/service_update/rki_import_get_new_dates_as_array')
@login_required
def url_rki_test_rki_service_service_update_rki_import_get_new_dates_as_array():
    app.logger.info("url_rki_test_rki_service_service_update_rki_import_get_new_dates_as_array - START")
    flash("url_rki_test_rki_service_service_update_rki_import_get_new_dates_as_array - START")
    for datum in rki_service.service_update.rki_import_get_new_dates_as_array():
        app.logger.info(str(datum))
    app.logger.info("url_rki_test_rki_service_service_update_rki_import_get_new_dates_as_array - DONE")
    flash("url_rki_test_rki_service_service_update_rki_import_get_new_dates_as_array - DONE")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/rki_test_service/delete_last_day')
@login_required
def url_rki_test_rki_test_service_delete_last_day():
    app.logger.info("url_rki_test_rki_test_service_delete_last_day - START: rki_test_service.delete_last_day()")
    flash("url_rki_test_rki_test_service_delete_last_day - START: rki_test_service.delete_last_day()")
    rki_test_service.delete_last_day()
    app.logger.info("url_rki_test_rki_test_service_delete_last_day - DONE: rki_test_service.delete_last_day()")
    flash("url_rki_test_rki_test_service_delete_last_day - DONE: rki_test_service.delete_last_day()")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/rki_test_service/delete_last_continent')
@login_required
def url_rki_test_rki_test_service_delete_last_continent():
    app.logger.info("url_rki_test_rki_test_service_delete_last_continent - START: rki_test_service.delete_last_continent()")
    flash("url_rki_test_rki_test_service_delete_last_continent - START: rki_test_service.delete_last_continent()")
    rki_test_service.delete_last_continent()
    flash("url_rki_test_rki_test_service_delete_last_continent - DONE: rki_test_service.delete_last_continent()")
    app.logger.info("url_rki_test_rki_test_service_delete_last_continent - DONE: rki_test_service.delete_last_continent()")
    return redirect(url_for('owid_test.url_rki_test_tests'))


@app_rki_test.route('/rki_test_service/full_update_dimension_tables')
@login_required
def url_rki_test_rki_test_service_full_update_dimension_tables():
    app.logger.info("url_rki_test_rki_test_service_full_update_dimension_tables - START: rki_test_service.full_update_dimension_tables()")
    flash("url_rki_test_rki_test_service_full_update_dimension_tables - START: rki_test_service.full_update_dimension_tables()")
    rki_test_service.full_update_dimension_tables()
    app.logger.info("url_rki_test_rki_test_service_full_update_dimension_tables - DONE: rki_test_service.full_update_dimension_tables()")
    flash("url_rki_test_rki_test_service_full_update_dimension_tables - DONE: rki_test_service.full_update_dimension_tables()")
    return redirect(url_for('owid_test.url_rki_test_tests'))


# ----------------------------------------------------------------------------------------------------------------
#  Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@celery.task(bind=True)
def task_rki_test_update_star_schema(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_rki_test_update_star_schema [OK] ")
    logger.info("------------------------------------------------------------")
    rki_test_service.run_update_star_schema_incremental()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_rki_test_update_star_schema)"
    return result


# ----------------------------------------------------------------------------------------------------------------
#  URL Routes for Celery TASKS
# ----------------------------------------------------------------------------------------------------------------

@app_rki_test.route('/task/owid_test/update_star_schema_incremental')
@login_required
def url_task_rki_test_update_star_schema():
    app.logger.info("url_task_rki_test_update_star_schema - START: task_rki_test_update_star_schema.apply_async()")
    flash("url_task_rki_test_update_star_schema - START: task_rki_test_update_star_schema.apply_async()")
    task_rki_test_update_star_schema.apply_async()
    flash("url_task_rki_test_update_star_schema - DONE: task_rki_test_update_star_schema.apply_async()")
    app.logger.info("url_task_rki_test_update_star_schema - DONE: task_rki_test_update_star_schema.apply_async()")
    return redirect(url_for('owid_test.url_rki_test_tests'))






