from flask import flash

from database import app
from covid19.blueprints.app_all.all_service_config import ApplicationServiceConfig
from covid19.blueprints.app_all.all_service_download import ApplicationServiceDownload
from covid19.blueprints.rki_vaccination.rki_vaccination_service_import import RkiVaccinationServiceImport
from covid19.blueprints.rki_vaccination.rki_vaccination_service_update import RkiVaccinationServiceUpdate, RkiVaccinationServiceUpdateFull


class RkiVaccinationService:
    def __init__(self, database):
        app.logger.debug("------------------------------------------------------------")
        app.logger.debug(" Vaccination Service [init]")
        app.logger.debug("------------------------------------------------------------")
        self.__database = database
        self.cfg = ApplicationServiceConfig.create_config_for_rki_vaccination()
        self.service_download = ApplicationServiceDownload(database, self.cfg)
        self.service_import = RkiVaccinationServiceImport(database, self.cfg)
        self.service_update = RkiVaccinationServiceUpdate(database, self.cfg)
        self.service_update_full = RkiVaccinationServiceUpdateFull(database, self.cfg)
        app.logger.debug("------------------------------------------------------------")
        app.logger.info(" Vaccination Service [ready]")

    def pretask_database_drop_create(self):
        flash("vaccination_service.run_download started")
        app.logger.info("vaccination_service.run_download started")
        self.service_download.download_file()
        app.logger.info("vaccination_service.run_download done")
        flash("vaccination_service.run_download done")
        return self

    def task_database_drop_create(self):
        app.logger.info("vaccination_service.task_database_drop_create started")
        self.service_import.import_file()
        self.service_update.update_star_schema_initial()
        app.logger.info("vaccination_service.task_database_drop_create done")
        return self

    def run_download(self):
        self.service_download.download_file()
        return self

    def run_import(self):
        self.service_import.import_file()
        return self

    def run_full_update_dimension_tables(self):
        self.service_update_full.update_dimension_tables_only()
        return self

    def run_update_dimension_tables(self):
        self.service_update.update_dimension_tables_only()
        return self

    def run_update_fact_table(self):
        self.service_update.update_fact_table_incremental_only()
        return self

    def run_full_update_fact_table(self):
        self.service_update_full.update_fact_table_initial_only()
        return self

    def task_admin_update_star_schema(self):
        self.service_update.update_star_schema_incremental()
        return self

    def task_admin_full_update_star_schema(self):
        self.run_import()
        self.service_update_full.update_star_schema_initial()
        return self

    def download_all_files(self):
        self.service_download.download_file()
        return self

    def task_admin_import_all_files(self):
        self.service_import.import_file()
        return self

    def task_admin_update_full_dimension_tables(self):
        self.service_update_full.update_dimension_tables_only()
        return self

    def task_admin_update_dimension_tables(self):
        self.service_update.update_dimension_tables_only()
        return self

    def update_dimension_tables(self):
        self.service_update.update_dimension_tables_only()
        return self

    def full_update_fact_table(self):
        self.service_update_full.update_fact_table_initial_only()
        return self

    def update_fact_table(self):
        self.service_update.update_fact_table_incremental_only()
        return self
