from flask import render_template, redirect, url_for, flash, Blueprint
from celery import states
from celery.utils.log import get_task_logger
from flask_login import login_required

from database import app, db
from covid19.blueprints.app_all.all_services import who_service
from covid19.blueprints.app_mq.application_workers import celery
from covid19.blueprints.app_application.application_model_transient import ApplicationPage

from covid19.blueprints.who.who_model_import import WhoImport
from covid19.blueprints.who.who_model import WhoData

from covid19.blueprints.who_test.who_test_service import WhoTestService

who_test_service = WhoTestService(db, who_service)

app_who_test = Blueprint('who_test', __name__, template_folder='templates', url_prefix='/who/test')

# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------


@app_who_test.route('/tests')
@login_required
def url_who_test_tests():
    page_info = ApplicationPage('WHO', "Tests")
    return render_template(
        'who_test/who_tests.html',
        page_info=page_info)


@app_who_test.route('/who_import/countries')
@login_required
def url_who_test_who_import_countries():
    flash("url_who_mytest - START: WhoImport.countries()")
    app.logger.info("url_who_mytest - START: WhoImport.countries()")
    i = 0
    for c in WhoImport.countries():
        i += 1
        line = " | " + str(i) + " | " + c.countries.country_code + " | " + c.countries.country + " | " + c.countries.who_region + " | "
        app.logger.info(line)
    flash("url_who_mytest - DONE: WhoImport.countries()")
    return redirect(url_for('who_test.url_who_test_tests'))


@app_who_test.route('/who_import/get_new_dates_as_array')
@login_required
def url_who_test_who_import_get_new_dates_as_array():
    app.logger.info("url_who_mytest - DONE: WhoImport.countries()")
    flash("url_who_mytest - START: WhoImport.get_new_dates_as_array()")
    app.logger.info("url_who_mytest - START: WhoImport.get_new_dates_as_array()")
    app.logger.info("WhoImport.get_new_dates_as_array():")
    i = 0
    for date_reported in WhoImport.get_new_dates_as_array():
        i += 1
        line = " | " + str(i) + " | " + date_reported + " | "
        app.logger.info(line)
    flash("url_who_mytest - DONE: WhoImport.get_new_dates_as_array()")
    app.logger.info("url_who_mytest - DONE: WhoImport.get_new_dates_as_array()")
    return redirect(url_for('who_test.url_who_test_tests'))


@app_who_test.route('/who_data/get_datum_of_all_who_data')
@login_required
def url_who_test_who_data_get_datum_of_all_who_data():
    app.logger.info("url_who_test_who_data_get_datum_of_all_who_data - DONE: WhoData.get_datum_of_all_who_data()")
    flash("url_who_test_who_data_get_datum_of_all_who_data - START: WhoData.get_datum_of_all_who_data()")
    for datum in WhoData.get_datum_of_all_data():
        app.logger.info(str(datum))
    flash("url_who_test_who_data_get_datum_of_all_who_data - DONE: WhoData.get_datum_of_all_who_data()")
    app.logger.info("url_who_test_who_data_get_datum_of_all_who_data - DONE: WhoData.get_datum_of_all_who_data()")
    return redirect(url_for('who_test.url_who_test_tests'))


@app_who_test.route('/who_data/get_datum_of_all_who_import')
@login_required
def url_who_test_who_data_get_datum_of_all_who_import():
    app.logger.info("url_who_test_who_data_get_datum_of_all_who_import - START: WhoImport.get_datum_of_all_who_import()")
    flash("url_who_test_who_data_get_datum_of_all_who_import - START: WhoImport.get_datum_of_all_who_import()")
    for datum in WhoImport.get_datum_of_all_who_import():
        app.logger.info(str(datum))
    flash("url_who_test_who_data_get_datum_of_all_who_import - DONE: WhoImport.get_datum_of_all_who_import()")
    app.logger.info("url_who_test_who_data_get_datum_of_all_who_import - DONE: WhoImport.get_datum_of_all_who_import()")
    return redirect(url_for('who_test.url_who_test_tests'))


@app_who_test.route('/who_service/service_update/who_import_get_new_dates_as_array')
@login_required
def url_who_test_who_service_who_import_get_new_dates_as_array():
    app.logger.info("url_who_test_who_import_get_new_dates_as_array - START: WhoService.who_import_get_new_dates_as_array()")
    flash("url_who_test_who_import_get_new_dates_as_array - START: WhoService.who_import_get_new_dates_as_array()")
    for datum in who_service.service_update.who_import_get_new_dates_as_array():
        app.logger.info(str(datum))
    flash("url_who_test_who_import_get_new_dates_as_array - DONE: WhoService.who_import_get_new_dates_as_array()")
    app.logger.info("url_who_test_who_import_get_new_dates_as_array - DONE: WhoService.who_import_get_new_dates_as_array()")
    return redirect(url_for('who_test.url_who_test_tests'))


@app_who_test.route('/who_test_service/delete_last_day')
@login_required
def url_who_test_who_test_service_delete_last_days_data():
    app.logger.info("url_who_test_who_test_service_delete_last_days_data - START: WhoService.who_import_get_new_dates_as_array()")
    flash("url_who_test_who_test_service_delete_last_days_data - START: WhoService.who_import_get_new_dates_as_array()")
    who_test_service.delete_last_day()
    flash("url_who_test_who_test_service_delete_last_days_data - DONE: WhoService.who_import_get_new_dates_as_array()")
    app.logger.info("url_who_test_who_test_service_delete_last_days_data - DONE: WhoService.who_import_get_new_dates_as_array()")
    return redirect(url_for('who_test.url_who_test_tests'))


# ----------------------------------------------------------------------------------------------------------------
#  Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@celery.task(bind=True)
def task_who_test_update_star_schema_incremental(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_who_test_update_star_schema_incremental [OK] ")
    logger.info("------------------------------------------------------------")
    who_test_service.run_update_star_schema_incremental()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_who_test_update_star_schema_incremental)"
    return result

# ----------------------------------------------------------------------------------------------------------------
#  URL Routes for Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@app_who_test.route('/task/update_star_schema_incremental')
@login_required
def url_task_who_test_update_star_schema_incremental():
    app.logger.info("url_task_who_update_star_schema_incremental - START: task_who_update_star_schema_incremental()")
    flash("url_task_who_update_star_schema_incremental - START: task_who_update_star_schema_incremental()")
    task_who_test_update_star_schema_incremental.apply_async()
    flash("url_task_who_update_star_schema_incremental - DONE: task_who_update_star_schema_incremental()")
    app.logger.info("url_task_who_update_star_schema_incremental - DONE: task_who_update_star_schema_incremental()")
    return redirect(url_for('who_test.url_who_test_tests'))
